/*

Youless powermeter clone for Esp8266.

Gebruikte info:
http://wiki.td-er.nl/index.php?title=YouLess

Meten van Wattvebruik van electricietsmeter door de periode tussen BLINKs van de LED te bapelen mbt van een BPX34 fototransistor
Aansluiten:
- collector aan D6        // wordt via interne pullup aan Vcc gehangen
- emitter aan GND

Interne LED van Wemos-D1-Mini zal het licht op de fototranssitor "volgen" als controle.

Gebruik:
- via webbrowser : http://192.168.0.32/a
  Response:
    84 kWh
    362 Watt
    0%
    OK (13)
    L:40 ms H:19817 ms
    Up 0d 0h 15m 36s
- via Domoticz, Youless is als default hardware bekend. Getest, werkt.

23-02-2017. Eerste versie o.b.v. Youless_powermeter_clone_ESP_interrupt. ISR verwijderd, op verdenking van verstoring Wifi verbinding. Werkt prima in de meterkast.

*/ 
 
#include <ESP8266WebServer.h>
ESP8266WebServer server(80);

#include <ESP8266WiFi.h>

#define PORT_DATA D6     // Wemos-D1-Mini D4 = ESP8266-GPIO2
#define PORT_GND D5     // Wemos-D1-Mini D5 = ESP8266-GPIO14

unsigned long durationHigh=0, durationLow=0;
unsigned long risingEdge=0;
unsigned long fallingEdge=0;
int state=0;

unsigned long watt=0;
unsigned long wh=0;

long timeout=0;


void checkPulse(){
  //Serial.print("*");
  int level = digitalRead(PORT_DATA);
  digitalWrite(BUILTIN_LED, level);
  
  unsigned long t = millis();
  switch (state) {
    case 0: //waiting for line to go HIGH
      if (level==1){ 
        // risingEdge
        risingEdge=t;
        durationLow=t-fallingEdge;
        state=1;
      }
      break;
    case 1: //waiting for line to go LOW
      if (level==0){ 
        // fallingEdge
        fallingEdge=t;
        durationHigh = t-risingEdge;
        
        if ( (durationLow > 10) && (durationLow < 100000  ) ){             // pulser < 100 msec and > 100000msec = 100sec are considered not valid
          // from msec to watt : Watt = 7 200 000 / Millislo  
          watt = 7200000 / (durationHigh+durationLow);
          // elke puls = 2Wh
          wh+=2;
           Serial.printf("L %d ms H %d ms L+H %d ms\n",durationLow, durationHigh, durationLow+durationHigh);       
          }
        else{
          durationLow=0;                                              // pulse of invalid length
        } // edn if/else
        state=0;
      }  // end if
      break;
  } // end switch
} // edn checkPulse

void setup()
{
  pinMode(BUILTIN_LED,OUTPUT);
  pinMode(PORT_DATA,INPUT_PULLUP);
  pinMode(PORT_GND,OUTPUT); digitalWrite(PORT_GND,LOW);

  Serial.begin(9600);	  // For debugging output
  connectWifi("","");
  // Start the server
  //server.on("/", handleRoot);

  server.on("/a", [](){
    // Prepare the response in plain text format
    char buf[100];

    unsigned long secs=millis()/1000, mins=secs/60;
    unsigned int hours=mins/60, days=hours/24;
    
    sprintf(buf, "%d kWh\r\n%d Watt\r\n0%\r\n\OK (13)\r\nL:%d ms H:%d ms\r\nUp %dd %dh %dm %ds\r\n",
      wh, watt, durationLow, durationHigh, days, hours % 24, mins % 60,secs % 60);    
    server.send(200, "text/plain", buf);
  });

  server.begin();
  Serial.println("HTTP server started");
} // end setup

void loop(){

  server.handleClient();    // check if a client wants attention

  checkPulse();             // check for a rising of falling pulse

}
